use std::path::Path;

use calibre_db::calibre::{Calibre, DataStoreError};
use snafu::{ResultExt, Snafu};

use crate::data::book::Book;

#[derive(Debug, Snafu)]
/// Errors that can occur when querying for books.
pub enum SearchQueryError {
    /// A failure to query the database.
    #[snafu(display("Failed to search for books."))]
    Db { source: DataStoreError },
}

/// Query for books and enrich them with additional information.
pub async fn query(
    query: &str,
    calibre: &Calibre,
    library_path: &Path,
) -> Result<Vec<Book>, SearchQueryError> {
    let books = calibre
        .search(query)
        .context(DbSnafu)?
        .iter()
        .filter_map(|book| Book::full_book(book, calibre, library_path))
        .collect();

    Ok(books)
}
