//! A very simple ebook server for a calibre library, providing a html interface as well as an OPDS feed.
//!
//! Shamelessly written to scratch my own itches.

use api::ServeError;
use app_state::AppState;
use calibre_db::calibre::{Calibre, LoadError};
use config::Config;
use snafu::{ResultExt, Snafu};
use tokio::signal;
use tracing::info;

pub mod api;
pub mod app_state;
pub mod cache;
pub mod cli;
pub mod config;
/// Data structs and their functions.
pub mod data;
/// OPDS data structs.
pub mod opds;
pub mod templates;

/// The application name.
const APP_NAME: &str = env!("CARGO_PKG_NAME");
/// The application version.
const APP_VERSION: &str = env!("CARGO_PKG_VERSION");

/// Internal marker data in lieu of a proper `Accept` header.
#[derive(Debug, Clone, Copy)]
pub enum Accept {
    /// Render as html.
    Html,
    /// Render as OPDS.
    Opds,
}

/// Errors from running little-hesinde.
#[derive(Debug, Snafu)]
pub enum RunError {
    #[snafu(display("Failed to load calibre database."))]
    LoadCalibre { source: LoadError },
    #[snafu(display("Failed to run http server."))]
    Serve { source: ServeError },
}

/// Main entry point to run the ebook server with a calibre library specified in `config`.
pub async fn run(config: Config) -> Result<(), RunError> {
    let calibre = Calibre::load(&config.metadata_path).context(LoadCalibreSnafu)?;
    let app_state = AppState {
        calibre,
        config: config.clone(),
    };

    let server = api::serve(config.listen_address, app_state);

    tokio::select! {
        res = server => {
            res.context(ServeSnafu)
        },
        _ = signal::ctrl_c() => {
            info!("Received Ctrl+C, shutting down...");
            Ok(())
        },
    }
}
