//! Relation data.

use serde_with::SerializeDisplay;

use super::media_type::MediaType;

/// Types of relations for links.
#[derive(Debug, SerializeDisplay)]
pub enum Relation {
    Image,
    /// Refer to the self feed.
    Myself,
    Start,
    Subsection,
    Thumbnail,
    Acquisition,
    Search,
}

/// Convert a media type int a relation.
///
/// This is not always deterministic but for the ones I actually use so far it is correct.
impl From<MediaType> for Relation {
    fn from(value: MediaType) -> Self {
        match value {
            MediaType::Acquisition => Relation::Acquisition,
            MediaType::Epub => Relation::Acquisition,
            MediaType::Html => Relation::Acquisition,
            MediaType::Jpeg => Relation::Image,
            MediaType::Navigation => Relation::Myself,
            MediaType::Pdf => Relation::Acquisition,
            MediaType::Text => Relation::Acquisition,
            MediaType::Search => Relation::Search,
        }
    }
}

/// Specify how to represent all relations in OPDS.
impl std::fmt::Display for Relation {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        match self {
            Relation::Image => write!(f, "http://opds-spec.org/image"),
            Relation::Myself => write!(f, "self"),
            Relation::Start => write!(f, "start"),
            Relation::Subsection => write!(f, "subsection"),
            Relation::Thumbnail => write!(f, "http://opds-spec.org/image/thumbnail"),
            Relation::Acquisition => write!(f, "http://opds-spec.org/acquisition"),
            Relation::Search => write!(f, "application/opensearchdescription+xml"),
        }
    }
}
